@@ -55,7 +55,7 @@ typedef struct {
  * @filename : in utf8.
  * @err	     : optionally NULL.
  *
- * Returns a new file or NULL.
+ * Returns: a new file or NULL.
  **/
 GsfInput *
 gsf_input_stdio_new (char const *filename, GError **err)
@@ -97,6 +97,11 @@ gsf_input_stdio_new (char const *filename, GError **err)
 
 	size = st.st_size;
 	input = (GsfInputStdio *)g_object_new (GSF_INPUT_STDIO_TYPE, NULL);
+	if (G_UNLIKELY (NULL == input)) {
+		fclose (file);
+		return NULL;
+	}
+
 	input->file = file;
 	input->filename = g_strdup (filename);
 	input->buf  = NULL;
@@ -117,8 +122,8 @@ gsf_input_stdio_new (char const *filename, GError **err)
  * Assumes ownership of @file.  If @keep_open is true, ownership reverts
  * to caller when the GsfObject is closed.
  *
- * Returns a new GsfInput wrapper for @file.  Note: the file must be
- * seekable, so this will not work for stdin when that is a tty or pipe.
+ * Returns: a new GsfInput wrapper for @file.  Note: the file must be
+ * 	seekable, so this will not work for stdin when that is a tty or pipe.
  **/
 GsfInput *
 gsf_input_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
@@ -138,6 +143,7 @@ gsf_input_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
 	size = st.st_size;
 
 	stdio = g_object_new (GSF_INPUT_STDIO_TYPE, NULL);
+	if (G_UNLIKELY (NULL == stdio)) return NULL;
 	stdio->file = file;
 	stdio->keep_open = keep_open;
 	stdio->filename = g_strdup (filename);
@@ -273,7 +279,7 @@ gsf_input_stdio_class_init (GObjectClass *gobject_class)
 	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
-GSF_DYNAMIC_CLASS (GsfInputStdio, gsf_input_stdio,
-		   gsf_input_stdio_class_init, gsf_input_stdio_init,
-		   GSF_INPUT_TYPE)
+GSF_CLASS (GsfInputStdio, gsf_input_stdio,
+	   gsf_input_stdio_class_init, gsf_input_stdio_init,
+	   GSF_INPUT_TYPE)
 
