@@ -2,7 +2,7 @@
 /*
  * gsf-output-stdio.c: stdio based output
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -77,48 +77,26 @@ rename_wrapper (char const *oldfilename, char const *newfilename)
 	return result;
 }
 
-/* g_access would be nice here.  */
-static gboolean
-gsf_access (char const *filename, int what)
+static int
+chmod_wrapper (const char *filename, mode_t mode)
 {
-#ifdef G_OS_WIN32
-	int retval;
-	int save_errno;
-	if (G_WIN32_HAVE_WIDECHAR_API ()) {
-		wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-		if (wfilename == NULL) {
-			errno = EINVAL;
-			return -1;
-		}
+#ifdef HAVE_G_CHMOD
+	return g_chmod (filename, mode);
+#else
+	return chmod (filename, mode);
+#endif
+}
 
-		retval = _waccess (wfilename, what);
-		save_errno = errno;
-
-		g_free (wfilename);
-      
-		errno = save_errno;
-	} else {
-		gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-		if (cp_filename == NULL) {
-			errno = EINVAL;
-			return -1;
-		}
-
-		retval = _access (cp_filename, what);
-		save_errno = errno;
-
-		g_free (cp_filename);
-
-		errno = save_errno;
-	}
-	return retval;
+static int
+access_wrapper (char const *filename, int what)
+{
+#ifdef HAVE_G_ACCESS
+	return g_access (filename, what);
 #else
 	return access (filename, what);
 #endif
 }
 
-
-
 #define GSF_MAX_LINK_LEVEL 256
 
 /* Calls g_file_read_link() until we find a real filename. */
@@ -168,159 +146,6 @@ follow_symlinks (char const *filename, GError **error)
 	return NULL;
 }
 
-/**
- * gsf_output_stdio_new :
- * @filename : in utf8.
- * @err	     : optionally NULL.
- *
- * Returns a new file or NULL.
- **/
-GsfOutput *
-gsf_output_stdio_new (char const *filename, GError **err)
-{
-	GsfOutputStdio *stdio;
-	FILE *file = NULL;
-	char *dirname = NULL;
-	char *temp_filename = NULL;
-	char *real_filename = follow_symlinks (filename, err);
-	int fd;
-	mode_t saved_umask;
-	struct stat st;
-	gboolean fixup_mode = FALSE;
-
-	if (real_filename == NULL)
-		goto failure;
-
-	/* Get the directory in which the real filename lives */
-	dirname = g_path_get_dirname (real_filename);
-
-	if (g_stat (real_filename, &st) == 0) {
-		if (!S_ISREG (st.st_mode)) {
-			if (err != NULL) {
-				char *dname = g_filename_display_name
-					(real_filename);
-				*err = g_error_new (gsf_output_error_id (), 0,
-						    "%s: Is not a regular file",
-						    dname);
-				g_free (dname);
-			}
-			goto failure;
-		}
-
-		/* FIXME? Race conditions en masse.  */
-		if (gsf_access (real_filename, W_OK) == -1) {
-			if (err != NULL) {
-				int save_errno = errno;
-				char *dname = g_filename_display_name
-					(real_filename);
-				*err = g_error_new
-					(gsf_output_error_id (), errno,
-					 "%s: %s",
-					 dname, g_strerror (save_errno));
-				g_free (dname);
-			}
-			goto failure;
-		}
-	} else {
-		/*
-		 * File does not exist.  Compute the permissions and uid/gid
-		 * that we will use for the newly-created file.
-		 */
-
-		struct stat dir_st;
-
-		memset (&st, 0, sizeof (st));
-
-		/* Use default permissions */
-		st.st_mode = 0666;  fixup_mode = TRUE;
-#ifdef HAVE_CHOWN
-		st.st_uid = getuid ();
-
-		if (g_stat (dirname, &dir_st) == 0 &&
-		    S_ISDIR (st.st_mode) &&
-		    (dir_st.st_mode & S_ISGID))
-			st.st_gid = dir_st.st_gid;
-		else
-			st.st_gid = getgid ();
-#endif
-	}
-
-	/* Save to a temporary file.  We set the umask because some (buggy)
-	 * implementations of mkstemp() use permissions 0666 and we want 0600.
-	 */
-	temp_filename = g_build_filename (dirname, ".gsf-save-XXXXXX", NULL);
-	/* Oh, joy.  What about threads?  --MW */
-	saved_umask = umask (0077);
-	fd = g_mkstemp (temp_filename); /* this modifies temp_filename to the used name */
-	umask (saved_umask);
-
-	if (fixup_mode)
-		st.st_mode &= ~saved_umask;
-
-	if (fd < 0 || NULL == (file = fdopen (fd, "wb"))) {
-		if (err != NULL) {
-			int save_errno = errno;
-			char *dname = g_filename_display_name
-				(temp_filename);
-			*err = g_error_new
-				(gsf_output_error_id (), errno,
-				 "%s: %s",
-				 dname, g_strerror (save_errno));
-			g_free (dname);
-		}
-		goto failure;
-	}
-
-	stdio = g_object_new (GSF_OUTPUT_STDIO_TYPE, NULL);
-	stdio->file = file;
-	stdio->st = st;
-	stdio->create_backup_copy = FALSE;
-	stdio->real_filename = real_filename;
-	stdio->temp_filename = temp_filename;
-
-	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
-
-	g_free (dirname);
-
-	return GSF_OUTPUT (stdio);
-
- failure:
-	g_free (temp_filename);
-	g_free (real_filename);
-	g_free (dirname);
-	return NULL;
-}
-
-/**
- * gsf_output_stdio_new_FILE :
- * @filename  : The filename corresponding to @file.
- * @file      : an existing stdio FILE *
- * @keep_open : Should @file be closed when the wrapper is closed
- *
- * Assumes ownership of @file.  If @keep_open is true, ownership reverts
- * to caller when the Gsfobject is closed.
- *
- * Returns a new GsfOutput wrapper for @file.  Warning: the result will be
- * seekable only if @file is seekable.  If it is seekable, the resulting
- * GsfOutput object will seek relative to @file's beginning, not its
- * current location at the time the GsfOutput object is created.
- **/
-GsfOutput *
-gsf_output_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
-{
-	GsfOutputStdio *stdio;
-
-	g_return_val_if_fail (filename != NULL, NULL);
-	g_return_val_if_fail (file != NULL, NULL);
-
-	stdio = g_object_new (GSF_OUTPUT_STDIO_TYPE, NULL);
-	stdio->file = file;
-	stdio->keep_open = keep_open;
-	stdio->real_filename = stdio->temp_filename = NULL;
-	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
-	return GSF_OUTPUT (stdio);
-}
-
 static gboolean
 close_file_helper (GsfOutputStdio *stdio, gboolean seterr)
 {
@@ -411,14 +236,15 @@ gsf_output_stdio_close (GsfOutput *output)
 		if (backup_filename != NULL &&
 		    rename_wrapper (backup_filename, stdio->real_filename) != 0)
 			saved_errno = errno;
-		res = gsf_output_set_error (output, errno, g_strerror (errno));
+		res = gsf_output_set_error (output,
+					    saved_errno,
+					    g_strerror (saved_errno));
 	} else {
 		/* Restore permissions.  There is not much error checking we
 		 * can do here, I'm afraid.  The final data is saved anyways.
 		 * Note the order: mode, uid+gid, gid, uid, mode.
 		 */
-#warning "We need g_chmod in gstdio.h for this"
-		chmod (stdio->real_filename, stdio->st.st_mode);
+		chmod_wrapper (stdio->real_filename, stdio->st.st_mode);
 #ifdef HAVE_CHOWN
 		if (chown (stdio->real_filename,
 			   stdio->st.st_uid,
@@ -427,7 +253,7 @@ gsf_output_stdio_close (GsfOutput *output)
 			chown (stdio->real_filename, -1, stdio->st.st_gid);
 			chown (stdio->real_filename, stdio->st.st_uid, -1);
 		}
-		chmod (stdio->real_filename, stdio->st.st_mode);
+		chmod_wrapper (stdio->real_filename, stdio->st.st_mode);
 #endif
 	}
 
@@ -520,6 +346,9 @@ gsf_output_stdio_write (GsfOutput *output,
 	return TRUE;
 }
 
+static gsf_off_t gsf_output_stdio_vprintf (GsfOutput *output,
+	char const *fmt, va_list args) G_GNUC_PRINTF (2, 0);
+
 static gsf_off_t
 gsf_output_stdio_vprintf (GsfOutput *output, char const *fmt, va_list args)
 {
@@ -551,4 +380,194 @@ gsf_output_stdio_class_init (GObjectClass *gobject_class)
 }
 
 GSF_CLASS (GsfOutputStdio, gsf_output_stdio,
-	   gsf_output_stdio_class_init, gsf_output_stdio_init, GSF_OUTPUT_TYPE)
+	   gsf_output_stdio_class_init, gsf_output_stdio_init,
+	   GSF_OUTPUT_TYPE)
+
+GsfOutput *
+gsf_output_stdio_new_valist (char const *filename, GError **err,
+			     char const *first_property_name,
+			     va_list     var_args)
+{
+	GsfOutputStdio *stdio;
+	FILE *file = NULL;
+	char *dirname = NULL;
+	char *temp_filename = NULL;
+	char *real_filename = follow_symlinks (filename, err);
+	int fd;
+	mode_t saved_umask;
+	struct stat st;
+	gboolean fixup_mode = FALSE;
+
+	if (real_filename == NULL)
+		goto failure;
+
+	/* Get the directory in which the real filename lives */
+	dirname = g_path_get_dirname (real_filename);
+
+	if (g_stat (real_filename, &st) == 0) {
+		if (!S_ISREG (st.st_mode)) {
+			if (err != NULL) {
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new (gsf_output_error_id (), 0,
+						    "%s: Is not a regular file",
+						    dname);
+				g_free (dname);
+			}
+			goto failure;
+		}
+
+		/* FIXME? Race conditions en masse.  */
+		if (access_wrapper (real_filename, W_OK) == -1) {
+			if (err != NULL) {
+				int save_errno = errno;
+				char *dname = g_filename_display_name
+					(real_filename);
+				*err = g_error_new
+					(gsf_output_error_id (), errno,
+					 "%s: %s",
+					 dname, g_strerror (save_errno));
+				g_free (dname);
+			}
+			goto failure;
+		}
+	} else {
+		/*
+		 * File does not exist.  Compute the permissions and uid/gid
+		 * that we will use for the newly-created file.
+		 */
+
+		memset (&st, 0, sizeof (st));
+
+		/* Use default permissions */
+		st.st_mode = 0666;  fixup_mode = TRUE;
+#ifdef HAVE_CHOWN
+		{
+			struct stat dir_st;
+
+			st.st_uid = getuid ();
+
+			if (g_stat (dirname, &dir_st) == 0 &&
+			    S_ISDIR (dir_st.st_mode) &&
+			    (dir_st.st_mode & S_ISGID))
+				st.st_gid = dir_st.st_gid;
+			else
+				st.st_gid = getgid ();
+		}
+#endif
+	}
+
+	/* Save to a temporary file.  We set the umask because some (buggy)
+	 * implementations of mkstemp() use permissions 0666 and we want 0600.
+	 */
+	temp_filename = g_build_filename (dirname, ".gsf-save-XXXXXX", NULL);
+	/* Oh, joy.  What about threads?  --MW */
+	saved_umask = umask (0077);
+	fd = g_mkstemp (temp_filename); /* this modifies temp_filename to the used name */
+	umask (saved_umask);
+
+	if (fixup_mode)
+		st.st_mode &= ~saved_umask;
+
+	if (fd < 0 || NULL == (file = fdopen (fd, "wb"))) {
+		if (err != NULL) {
+			int save_errno = errno;
+			char *dname = g_filename_display_name
+				(temp_filename);
+			*err = g_error_new
+				(gsf_output_error_id (), errno,
+				 "%s: %s",
+				 dname, g_strerror (save_errno));
+			g_free (dname);
+		}
+		goto failure;
+	}
+
+	stdio = (GsfOutputStdio *)g_object_new_valist (GSF_OUTPUT_STDIO_TYPE,
+		first_property_name, var_args);
+	stdio->file = file;
+	stdio->st = st;
+	stdio->create_backup_copy = FALSE;
+	stdio->real_filename = real_filename;
+	stdio->temp_filename = temp_filename;
+
+	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
+
+	g_free (dirname);
+
+	return GSF_OUTPUT (stdio);
+
+ failure:
+	g_free (temp_filename);
+	g_free (real_filename);
+	g_free (dirname);
+	return NULL;
+}
+
+/**
+ * gsf_output_stdio_new_full :
+ * @filename : name of file to create or replace.
+ * @err	     : optionally %NULL.
+ * @first_property_name : %NULL terminated list of properties
+ * @Varargs : 
+ *
+ * Returns: a new file or %NULL.
+ **/
+GsfOutput *
+gsf_output_stdio_new_full (char const *filename, GError **err,
+			   char const *first_property_name, ...)
+{
+	GsfOutput *res;
+	va_list var_args;
+	
+	va_start (var_args, first_property_name);
+	res = gsf_output_stdio_new_valist (filename, err, first_property_name, var_args);
+	va_end (var_args);
+
+	return res;
+}
+
+/**
+ * gsf_output_stdio_new :
+ * @filename : name of file to create or replace.
+ * @err	     : optionally %NULL.
+ *
+ * Returns: a new file or %NULL.
+ **/
+GsfOutput *
+gsf_output_stdio_new (char const *filename, GError **err)
+{
+	return gsf_output_stdio_new_full (filename, err, NULL);
+}
+
+/**
+ * gsf_output_stdio_new_FILE :
+ * @filename  : The filename corresponding to @file.
+ * @file      : an existing stdio FILE *
+ * @keep_open : Should @file be closed when the wrapper is closed
+ *
+ * Assumes ownership of @file.  If @keep_open is true, ownership reverts
+ * to caller when the GsfObject is closed.
+ *
+ * Returns: a new GsfOutput wrapper for @file.  Warning: the result will be
+ * seekable only if @file is seekable.  If it is seekable, the resulting
+ * GsfOutput object will seek relative to @file's beginning, not its
+ * current location at the time the GsfOutput object is created.
+ **/
+GsfOutput *
+gsf_output_stdio_new_FILE (char const *filename, FILE *file, gboolean keep_open)
+{
+	GsfOutputStdio *stdio;
+
+	g_return_val_if_fail (filename != NULL, NULL);
+	g_return_val_if_fail (file != NULL, NULL);
+
+	stdio = g_object_new (GSF_OUTPUT_STDIO_TYPE, NULL);
+	if (G_UNLIKELY (NULL == stdio)) return NULL;
+
+	stdio->file = file;
+	stdio->keep_open = keep_open;
+	stdio->real_filename = stdio->temp_filename = NULL;
+	gsf_output_set_name_from_filename (GSF_OUTPUT (stdio), filename);
+	return GSF_OUTPUT (stdio);
+}
