@@ -2,7 +2,7 @@
 /*
  * gsf-output.c: interface for storing data
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -167,9 +167,11 @@ GSF_CLASS_ABSTRACT (GsfOutput, gsf_output,
 
 /**
  * gsf_output_name :
- * @output :
+ * @output: #GsfOutput
  *
- * Returns @output's name in utf8 form, DO NOT FREE THIS STRING
+ * Give the name of @output.
+ *
+ * Returns: @output's name in utf8 form, DO NOT FREE THIS STRING
  **/
 char const *
 gsf_output_name (GsfOutput const *output)
@@ -182,8 +184,8 @@ gsf_output_name (GsfOutput const *output)
  * gsf_output_container :
  * @output :
  *
- * Returns, but does not add a reference to @output's container.
- * Potentially NULL
+ * Returns: but does not add a reference to @output's container.
+ * 	Potentially %NULL
  **/
 GsfOutfile *
 gsf_output_container (GsfOutput const *output)
@@ -194,9 +196,11 @@ gsf_output_container (GsfOutput const *output)
 
 /**
  * gsf_output_size :
- * @output :
+ * @output: #GsfOutput
  *
- * Returns the size of the output, or -1 if it does not have a size.
+ * Determine the size of the output stream @output.
+ *
+ * Returns: the size of the output, or -1 if it does not have a size.
  **/
 gsf_off_t
 gsf_output_size (GsfOutput *output)
@@ -207,10 +211,11 @@ gsf_output_size (GsfOutput *output)
 
 /**
  * gsf_output_close :
- * @output :
+ * @output:
  *
  * Close a stream.
- * Returns FALSE on error
+ *
+ * Returns: %FALSE on error
  **/
 gboolean
 gsf_output_close (GsfOutput *output)
@@ -232,9 +237,9 @@ gsf_output_close (GsfOutput *output)
 
 /**
  * gsf_output_is_closed :
- * @output :
+ * @output:
  *
- * Returns TRUE if @output has already been closed.
+ * Returns: %TRUE if @output has already been closed.
  **/
 gboolean
 gsf_output_is_closed (GsfOutput const *output)
@@ -245,9 +250,13 @@ gsf_output_is_closed (GsfOutput const *output)
 
 /**
  * gsf_output_tell :
- * @output :
+ * @output : #GsfOutput
  *
- * Returns the current position in the file
+ * Tell the current position in @output, similar to
+ * <citerefentry><refentrytitle>ftell</refentrytitle>
+ * <manvolnum>3</manvolnum></citerefentry>.
+ *
+ * Returns: the current position in the file
  **/
 gsf_off_t
 gsf_output_tell	(GsfOutput *output)
@@ -259,11 +268,18 @@ gsf_output_tell	(GsfOutput *output)
 
 /**
  * gsf_output_seek :
- * @output :
- * @offset :
- * @whence :
+ * @output : #GsfOutput
+ * @offset : Relative amount to reposition
+ * @whence : What the offset is relative to.
  *
- * Returns FALSE on error.
+ * Reposition in output stream @output. @whence specifies what the offset is
+ * relative to: the beginning of the stream (%G_SEEK_SET), current position in
+ * the stream (%G_SEEK_CUR) or the end of the stream (%G_SEEK_END).
+ * This function is similar to 
+ * <citerefentry><refentrytitle>fseek</refentrytitle>
+ * <manvolnum>3</manvolnum></citerefentry>.
+ *
+ * Returns: %FALSE on error.
  **/
 gboolean
 gsf_output_seek	(GsfOutput *output, gsf_off_t offset, GSeekType whence)
@@ -319,11 +335,13 @@ gsf_output_inc_cur_offset (GsfOutput *output, gsf_off_t num_bytes)
 
 /**
  * gsf_output_write :
- * @output :
- * @num_bytes :
- * @data :
+ * @output : Output stream
+ * @num_bytes : Number of bytes to write
+ * @data : Data to write.
  *
- * Returns FALSE on error.
+ * Write @num_bytes of @data to @output.
+ *
+ * Returns: %FALSE on error.
  **/
 gboolean
 gsf_output_write (GsfOutput *output,
@@ -342,9 +360,9 @@ gsf_output_write (GsfOutput *output,
 
 /**
  * gsf_output_error :
- * @output :
+ * @output:
  *
- * Returns the last error logged on the output, or NULL.
+ * Returns: the last error logged on the output, or %NULL.
  **/
 GError const *
 gsf_output_error (GsfOutput const *output)
@@ -355,13 +373,13 @@ gsf_output_error (GsfOutput const *output)
 
 /**
  * gsf_output_set_name :
- * @output :
- * @name :
+ * @output:
+ * @name:
  *
  * <note>This is a utility routine that should only be used by derived
  * outputs.</note>
  *
- * Returns : TRUE if the assignment was ok.
+ * Returns: %TRUE if the assignment was ok.
  **/
 gboolean
 gsf_output_set_name (GsfOutput *output, char const *name)
@@ -384,7 +402,7 @@ gsf_output_set_name (GsfOutput *output, char const *name)
  * <note>This is a utility routine that should only be used by derived
  * outputs.</note>
  *
- * Returns : TRUE if the assignment was ok.
+ * Returns: %TRUE if the assignment was ok.
  **/
 gboolean
 gsf_output_set_name_from_filename (GsfOutput *output, char const *filename)
@@ -400,13 +418,13 @@ gsf_output_set_name_from_filename (GsfOutput *output, char const *filename)
 
 /**
  * gsf_output_set_container :
- * @output :
- * @container :
+ * @output:
+ * @container:
  *
  * <note>This is a utility routine that should only be used by derived
  * outputs.</note>
  *
- * Returns : TRUE if the assignment was ok.
+ * Returns: %TRUE if the assignment was ok.
  **/
 gboolean
 gsf_output_set_container (GsfOutput *output, GsfOutfile *container)
@@ -423,15 +441,15 @@ gsf_output_set_container (GsfOutput *output, GsfOutfile *container)
 
 /**
  * gsf_output_set_error :
- * @output :
- * @code :
- * @format :
- * @Varargs :
+ * @output:
+ * @code:
+ * @format:
+ * @Varargs:
  *
  * <note>This is a utility routine that should only be used by derived
  * outputs.</note>
  *
- * Returns Always returns FALSE to facilitate its use.
+ * Returns: Always returns %FALSE to facilitate its use.
  **/
 gboolean
 gsf_output_set_error (GsfOutput  *output,
@@ -464,10 +482,10 @@ cb_output_unwrap (GsfOutput *wrapee, G_GNUC_UNUSED GObject *wrapper)
 
 /**
  * gsf_output_wrap :
- * @wrapper :
- * @wrapee :
+ * @wrapper:
+ * @wrapee:
  *
- * Returns TRUE if the wrapping succeeded.
+ * Returns: %TRUE if the wrapping succeeded.
  **/
 gboolean
 gsf_output_wrap (GObject *wrapper, GsfOutput *wrapee)
@@ -488,10 +506,10 @@ gsf_output_wrap (GObject *wrapper, GsfOutput *wrapee)
 
 /**
  * gsf_output_unwrap :
- * @wrapper :
- * @wrapee :
+ * @wrapper:
+ * @wrapee:
  *
- * Returns TRUE if the unwrapping succeeded.
+ * Returns: %TRUE if the unwrapping succeeded.
  **/
 gboolean
 gsf_output_unwrap (GObject *wrapper, GsfOutput *wrapee)
@@ -515,12 +533,16 @@ gsf_output_error_id (void)
 }
 
 /**
- * gsf_output_printf:
- * @output: A #GsfOutput
- * @format: The printf-style format string
- * @Varargs: the arguments for @format
+ * gsf_output_printf :
+ * @output : A #GsfOutput
+ * @format : The printf-style format string
+ * @Varargs : the arguments for @format
  *
- * Returns: TRUE if successful, FALSE if not
+ * Output @Varargs to @output using the format string @format, similar to
+ * <citerefentry><refentrytitle>printf</refentrytitle>
+ * <manvolnum>3</manvolnum></citerefentry>.
+ *
+ * Returns: %TRUE if successful, %FALSE if not
  **/
 gboolean
 gsf_output_printf (GsfOutput *output, char const *format, ...)
@@ -535,10 +557,14 @@ gsf_output_printf (GsfOutput *output, char const *format, ...)
 }
 
 /**
- * gsf_output_vprintf:
- * @output: A #GsfOutput
- * @format: The printf-style format string
- * @args: the arguments for @format
+ * gsf_output_vprintf :
+ * @output : A #GsfOutput
+ * @format : The printf-style format string
+ * @args : the arguments for @format
+ *
+ * Output @args to @output using the format string @format, similar to
+ * <citerefentry><refentrytitle>vprintf</refentrytitle>
+ * <manvolnum>3</manvolnum></citerefentry>.
  *
  * Returns: number of bytes printed, a negative value if not successful
  **/
@@ -563,6 +589,13 @@ static gsf_off_t
 gsf_output_real_vprintf (GsfOutput *output, char const *fmt, va_list args)
 {
 	gsf_off_t reslen;
+	va_list args2;
+
+	/*
+	 * We need to make a copy as args will become unusable after
+	 * the g_vsnprintf call.
+	 */
+	G_VA_COPY (args2, args);
 
 	if (NULL == output->printf_buf) {
 		output->printf_buf_size = 128;
@@ -573,7 +606,7 @@ gsf_output_real_vprintf (GsfOutput *output, char const *fmt, va_list args)
 	/* handle C99 or older -1 case of vsnprintf */
 	if (reslen < 0 || reslen >= output->printf_buf_size) {
 		g_free (output->printf_buf);
-		output->printf_buf = g_strdup_vprintf (fmt, args);
+		output->printf_buf = g_strdup_vprintf (fmt, args2);
 		reslen = output->printf_buf_size = strlen (output->printf_buf);
 	}
 
@@ -591,7 +624,7 @@ gsf_output_real_vprintf (GsfOutput *output, char const *fmt, va_list args)
  *
  * Like fputs, this assumes that the line already ends with a newline 
  *
- * Returns: TRUE if successful, FALSE if not
+ * Returns: %TRUE if successful, %FALSE if not
  **/
 gboolean
 gsf_output_puts (GsfOutput *output, char const *line)
